#include <capdev_file.h>
#include <s_util.h>

PLUGIN_DECLARE_STD_IMPORTS;
proc_add_captype add_captype;

void *lfile_init(char *interface, char *filter)
{
	struct pcap_file_header fh;
	p_cfile_priv p=my_malloc(sizeof(cfile_priv_t));

	if (p == NULL) return NULL;

	/* TODO: Open libpcap file */
	if ( (p->fd=open(interface, O_RDONLY)) < 0 ) {
		my_free(p);
		print_out(PRINT_ERROR "File %s not found" CRLF, interface);
		return NULL;
	}

	
	if ( (read(p->fd, (char *)&fh, sizeof(fh))) < sizeof(fh) ) {
		close(p->fd);
		p->fd=-1;
		my_free(p);
		print_out(PRINT_ERROR "Error with pcap header in %s" CRLF,
			interface);
		return NULL;
	}

	if ( fh.snaplen > CAP_SNAPLEN ) {
		print_out(PRINT_WARNING "Libpcap file %s has snaplen bigger "
			"than I can allow!" CRLF, interface);
	}

#ifdef __PLUGIN_DEBUG__
	print_out ( "LFILE: magic=%x" CRLF, fh.magic );
	print_out ( "LFILE: major=%i" CRLF, fh.version_major);
	print_out ( "LFILE: minor=%u" CRLF, fh.version_minor);
	print_out ( "LFILE: splen=%u" CRLF, fh.snaplen);
#endif
	
	if ( fh.magic == 0xa1b2cd34 && sizeof(struct pcap_pkthdr) != 24)  {
		print_raw(PRINT_WARNING "LBL libpcap, RedHat file. "
			"Engaging fixup." CRLF);
		p->fix = FIXUP_LBL;
	}else if ( fh.magic == 0xa1b2c3d4 && sizeof(struct pcap_pkthdr) == 24){
		print_raw(PRINT_WARNING "RedHat libpcap, LBL file. "
			"Engaging fixup." CRLF);
		p->fix = FIXUP_REDHAT;
	}
	
	p->snaplen=fh.snaplen;
	p->link=fh.linktype;
	
	return p;	
}

int lfile_end(void *priv)
{
	p_cfile_priv p=(p_cfile_priv)priv;
	if ( p == NULL ) return 0;
	close(p->fd);
	my_free(p);
	return 1;
}

struct packet *lfile_poll(void *priv)
{
	struct pcap_pkthdr h;
	uint len=0;
	uint siz=sizeof(h);
	
	p_cfile_priv p=(p_cfile_priv)priv;
	if ( p == NULL ) return NULL;
	
	if ( p->fix == FIXUP_REDHAT ) {
		siz += p->fix;
	}
	
	if ( (read(p->fd, (char *)&h, siz)) < siz ) {
		/* EOF */
		return NULL;
	}

	if ( p->fix == FIXUP_LBL ) {
		lseek(p->fd, p->fix, SEEK_CUR);
	}
	/* Make sure we dont overrun buffer */
	if ( h.caplen > CAP_SNAPLEN ) {
		len=CAP_SNAPLEN;
		lseek(p->fd, h.caplen - len, SEEK_CUR);
		print_out(PRINT_WARNING "Fixed up packet by %i" CRLF, 
			h.caplen - len);
	}else{
		len=h.caplen;
	}

	
	memset(&p->pkt, 0, sizeof(p->pkt));
	p->pkt.time = h.ts.tv_sec;
	p->pkt.nano_time = h.ts.tv_usec;
	if ( read(p->fd, (char *)&p->pkt.data, len) < len ) {
		print_raw("Bugger, short read" CRLF);
		return NULL;
	}
	
	p->pkt.caplen = len;
	p->pkt.len = h.len;
	
	p->pkt.layer[LAYER_LINK].id=p->link;
	
	/* Sleep is damn dangerous segfaults when
	* parent gets a SIGINT  */
	//sleep(1);
	
	return &p->pkt;
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);
	PLUGIN_IMPORT_STD;
	
	add_captype=in->add_captype;
	
	out->name="capture/libpcap_file";
	out->desc="Capture packets from libpcap (tcpdump) files";
	out->ver_major=1;
	out->ver_minor=0;

	if ( add_captype("libpcap_file", lfile_init, lfile_end, lfile_poll) ) {
		return PLUGIN_ERR_OK;
	}else{
		return PLUGIN_ERR_FAIL;
	}
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
