#include <capdev_netlink.h>
#include <s_util.h>

PLUGIN_DECLARE_STD_IMPORTS;

proc_add_captype add_captype;

void *lnl_init(char *interface, char *filter)
{
	p_cnl_priv p;

	if ( !(p=my_malloc(sizeof(cnl_priv_t))) ) return NULL;

	/* Create a firewall netlink socket */
	if ( (p->fd=socket(PF_NETLINK, SOCK_RAW, NETLINK_FIREWALL)) < 0 ) {
		my_free(p);
		return NULL;
	}

	memset( &p->nls, 0, sizeof(p->nls));
	p->nls.nl_family = AF_NETLINK;
	p->nls.nl_groups = -1;

	if ( bind(p->fd, (struct sockaddr *)&p->nls, sizeof(p->nls)) ) {
		my_free(p);
		return NULL;
	}
	
	if ( strcasecmp(interface, "any") ) {
		p->use_if=1;
		strcpy((char *)&p->iface, interface);
	}else{
		p->use_if=0;
		memset((char *)&p->iface, 0, IFNAMSIZ);
	}
	
	return p;
}

int lnl_end(void *priv)
{
	p_cnl_priv	p=priv;
	
	if ( !priv ) return 0;

	close(p->fd);
	
	return 1;
}

struct packet *lnl_poll(void *priv)
{
	nl_temp_t	temp;
	p_cnl_priv	p=priv;

	if ( p == NULL ) return NULL;
	if ( p->fd < 0) return NULL;
	
	memset(&p->pkt, 0, sizeof(p->pkt));
	
	/* EEK */
	while ( recv(p->fd, &temp, sizeof(temp), 0) <= 0 ) {
		if ( p->use_if == 0 ) 
			break;
		
		if ( strcmp(p->iface, temp.hdr.iface) == 0  )
			break;
		
		continue;
	}
	
	/* Get the time */
	time(&p->pkt.time);
	
	p->pkt.len = temp.hdr.len;
	p->pkt.caplen = temp.hdr.len;
	memcpy(&p->pkt.data, &temp.data, CAP_SNAPLEN);
	
	return &p->pkt;
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);
	PLUGIN_IMPORT_STD;
	
	add_captype=in->add_captype;
	
	out->name="capture/netlink";
	out->desc="Capture packets from linux firewall netlink";
	out->ver_major=1;
	out->ver_minor=0;

	if ( add_captype("netlink", lnl_init, lnl_end, lnl_poll) ) {
		return PLUGIN_ERR_OK;
	}else{
		return PLUGIN_ERR_FAIL;
	}
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
