#include <match_std.h>
#include <s_util.h>

PLUGIN_DECLARE_STD_IMPORTS;

proc_matcher_add	add_matcher;

void *proto_match_validate(char *token, p_kv opts)
{
	p_num_priv priv=(p_num_priv)token;
	struct protoent *pr;
	
	if ( !token ) return NULL;

	pr = getprotobyname(token);
	if ( !pr ) {
		print_out("IP protocol '%s' not known" CRLF, token);
		return NULL;
	}

	if ( !(priv=my_malloc(sizeof(num_priv_t))) ) {
		return NULL;
	}
	
	priv->val = pr->p_proto;
	return priv;
}

void *ipaddr_validate(char *token, p_kv opts)
{
	char *ip;
	char *tmp;
	char *addr, *mask;
	p_ipaddr_priv priv;
	struct in_addr id;

	
	if (!token) return NULL;
	
	if ( !(priv=my_malloc(sizeof(ipaddr_priv_t))) ) return NULL;
	memset(priv, 0, sizeof(ipaddr_priv_t));

	/* Its good manners to make a copy of token
	* cos our parsing will clobber it */
	if ( !(ip=my_strdup(token)) ) return NULL;
	tmp=ip; /* ip can be modified by strsep */
	
	if ( strchr(ip, '/') ) {
		addr=strsep(&ip, "/");
		mask=strsep(&ip, "/");
		priv->mask=GEN_IPMASK(atoi(mask));
	}else{
		addr=ip;
		priv->mask=-1;
	}

	priv->addr=inet_addr(addr);
	if ( priv->addr == -1 ) {
		my_free(priv);
		my_free(tmp);
		return NULL;
	}
	
	my_free(tmp);
	return priv;
}

int proto_match(void *priv, p_packet p, int negate)
{
	p_num_priv x=(p_num_priv)priv;

	if ( x->val == 0 ) return 0;

	/* Check its actually IP */
	if ( p->layer[LAYER_NET].id != 0x0800 ) return 0;

	if ( p->layer[LAYER_NET].h.ip->protocol == x->val ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}

int ipaddr_match_dst(void *priv, p_packet p, int negate)
{
	uint s,d;
	p_ipaddr_priv x;

	if ( !(x=(p_ipaddr_priv)priv) ) return 0;
	if ( p->layer[LAYER_NET].id != 0x0800 ) return 0;

	s=p->layer[LAYER_NET].h.ip->daddr & x->mask;
	d=x->addr & x->mask;

	if (s==d) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}

int ipaddr_match_src(void *priv, p_packet p, int negate)
{
	uint s,d;
	p_ipaddr_priv x;
	
	if ( !(x=(p_ipaddr_priv)priv) ) return 0;
	if ( p->layer[LAYER_NET].id != 0x0800 ) return 0;
	
	s=p->layer[LAYER_NET].h.ip->saddr & x->mask;
	d=x->addr & x->mask;

	if (s==d) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}

int dsize_match(void *priv, p_packet p, int negate)
{
	p_range_priv x=(p_range_priv)priv;

	if ( (p->caplen >= x->min) && (p->caplen <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}

int port_match_src(void *priv, p_packet p, int negate)
{
	p_range_priv x=(p_range_priv)priv;
	if ( !priv || !p ) return 0;

	if ( (p->sport >= x->min) && (p->sport <= x->max) ) {
		MATCH_RET_POSITIVE;
	}

	MATCH_RET_NEGATIVE;
}

int port_match_dst(void *priv, p_packet p, int negate)
{
	p_range_priv x=(p_range_priv)priv;
	if (!priv || !p ) return 0;
	
	if ( (p->dport >= x->min) && (p->dport <= x->max) ) {
		MATCH_RET_POSITIVE;
	}

	MATCH_RET_NEGATIVE;
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);
	PLUGIN_IMPORT_STD;
	
	add_matcher=in->add_matcher;

	out->name="match/standard";
	out->desc="Matches IP addresses and ports";
	out->ver_major=1;
	out->ver_minor=0;
	
	add_matcher("SPT", 
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		port_match_src);

	add_matcher("DPT",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		port_match_dst);
	
	add_matcher("IP_PROTO",
		proto_match_validate,
		in->templates.t_cleanup_ptr,
		proto_match);
	
	add_matcher("IP_SRC",
		ipaddr_validate,
		in->templates.t_cleanup_ptr,
		ipaddr_match_src);
	
	add_matcher("IP_DST",
		ipaddr_validate,
		in->templates.t_cleanup_ptr,
		ipaddr_match_dst);
	
	add_matcher("ITYPE",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		port_match_src);
	
	add_matcher("ICODE",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		port_match_dst);
	
	add_matcher("DSIZE",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		dsize_match);
	
	return PLUGIN_ERR_OK;
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
