#include <demux.h>
#include <s_util.h>
#include <cleanup.h>

p_demux_layer	dlist[LAYER_MAX];
mlock_t		dlock;;

/* go through our linked list of demux handlers
 * and try to demultiplex this packet */
int demux_layer_from_plugins(int layer, p_packet p)
{
	p_demux_layer d=NULL;
 
	CRITICAL_START(dlock);
	for (d=dlist[layer]; d; d=(p_demux_layer)d->l.next )
	{
		switch ( d->demux(p) ) {
			
			case DEMUX_SKIP:
				CRITICAL_STOP(dlock);
				return DLFP_OK;
				
			case DEMUX_JUMP:
				CRITICAL_STOP(dlock);
				return DLFP_JUMP;

			case DEMUX_DROP:
				CRITICAL_STOP(dlock);
				return DLFP_JUMP;

			case DEMUX_NULL:
				break;
		}
	}

	CRITICAL_STOP(dlock);
	return DLFP_NOT_FOUND;
}

network_layer_t demux ( p_packet p )
{
	int retval = 0;

	p->layer[LAYER_LINK].h.raw = &p->data;

	for (retval=0; retval<LAYER_MAX; retval++) {
		switch ( demux_layer_from_plugins(retval, p) ) {
			
			case DLFP_NOT_FOUND:
				//print_out("Layer %i type (%i) not supported."
				//	CRLF, retval, p->layer[retval].id);
				return retval;
				
			case DLFP_OK:
				break;

			case DLFP_JUMP:
				//return retval;
				return LAYER_TRANS;

			case DLFP_DROP:
				print_out("Plugin forced me to drop a packet "
					"at layer %i (?!)" CRLF, retval);

			case DLFP_ERROR:
				print_out("Plugin caused demux() error at "
					"layer %i" CRLF, retval);
				return retval;
				
		}
	}
	
	return LAYER_LINK;
}

/* Cleanup our lists */
void demux_cleanup(int code, void *priv)
{
	p_demux_layer tmp=NULL, foo=NULL;
	int i;
	
	print_raw("Cleaning up demux() handlers" CRLF);
	CRITICAL_START(dlock);
	for (i=0; i<LAYER_MAX; i++) {
		for (tmp=dlist[i]; tmp;) {
			foo=tmp;
			tmp=(p_demux_layer)tmp->l.next;
			globals.free(foo);
			foo=NULL;
		}
	}
	CRITICAL_STOP(dlock);
	return;
}

int demux_init(void)
{
	int i;

	MUTEX_CREATE(dlock, "Demux");
	cleanup_add_handler(cleanup_lock, (void *)&dlock);

	
	CRITICAL_START(dlock);
	for (i=0; i<LAYER_MAX; i++){
		dlist[i]=NULL;
	}
	CRITICAL_STOP(dlock);

	cleanup_add_handler(demux_cleanup, NULL);

	return 1;
}

uint demux_add_handler (uint layer, proc_demux target)
{
	p_demux_layer tmp=NULL;
	
	if ( layer > LAYER_MAX ) {
		print_out(PRINT_ERROR "DEMUX: plugin tried to add layer "
			"%i handler (?!)" CRLF, layer);
		return 0;
	}

	if ( !(tmp=globals.malloc(sizeof(demux_layer_t))) ) {
		print_raw(PRINT_ERROR "DEMUX: Ran OOM allocating "
			"demux handler" CRLF);
		return 0;
	}

	/* Actually add it */
	tmp->demux = target;
	CRITICAL_START(dlock);
	dlist[layer]=list_append(tmp, dlist[layer]);
	CRITICAL_STOP(dlock);

	return 1;
}
