#include <mem.h>
#include <cleanup.h>

#ifdef LEAK_CHECK

mlock_t memlock;

/* Protected by memlock */
u_int32_t mem_alloc=0;
u_int32_t mem_free=0;
u_int32_t mem_badfree=0;
p_mem allocs=NULL;

int mem_init(void)
{
	MUTEX_CREATE(memlock, "memory tracker");
	return 1;
}

char *fs_strdup(char *s)
{
	char *ret;

	if ( !s ) return NULL;
	
	if ( !(ret=fs_alloc(strlen(s)+1)) ) return NULL;
	
	strcpy(ret, s);
	return ret;
}

void *fs_alloc(size_t size)
{
	p_mem x;
	void *ret;
	
	if ( !(x=malloc(sizeof(mem_t))) ) return NULL;
	if ( !(ret=malloc(size)) ) {
		free(x);
		return NULL;
	}

	x->size=size;
	x->ptr=ret;
	
	CRITICAL_START(memlock);
	mem_alloc+=size;
	allocs=list_append(x, allocs);
	CRITICAL_STOP(memlock);
	
	return ret;
}

void fs_free(void *ptr)
{
	p_mem x;

	if ( !ptr ) return;
	
	CRITICAL_START(memlock);
	for ( x=allocs; x; x=(p_mem)x->l.next) {
		if ( x->ptr == ptr ) {
			mem_free += x->size;
			if ( x->l.prev == NULL ) allocs = x->l.next;
			free(list_remove(x));
			CRITICAL_STOP(memlock);
			free(ptr);
			return;
		}
	}
	mem_badfree++;
	CRITICAL_STOP(memlock);
	free(ptr);
}

void fs_usage(void)
{
	print_out("Currently using %i bytes (%i KB) of memory" CRLF, 
		mem_alloc-mem_free,
		(mem_alloc-mem_free)/1024);
}

void fs_memstats(void)
{
	CRITICAL_START(memlock);
	print_raw(CRLF "--" CRLF);
	print_out("Memory allocated: %i bytes" CRLF, mem_alloc);
	print_out("Memory freed:     %i bytes" CRLF, mem_free);
	print_out("Leakage:          %i bytes" CRLF, mem_alloc-mem_free);
	print_out("Bad freed blocks: %i blocks" CRLF, mem_badfree);
	print_raw("--" CRLF);
	CRITICAL_STOP(memlock);
}

#endif
