#include <s_globals.h>
#include <rules.h>
#include <cleanup.h>

p_parser parsers=NULL;
mlock_t  plock;

int rules_loadfile(char *type, char *fn)
{
	p_parser l;

	if ( !type || !fn ) return 0;

	for (l=parsers; l; l=(p_parser)l->l.next) {
		if ( strcasecmp(l->name, type) == 0 ) {
			if ( l->loadfile(fn) ){
				globals.num_rulefiles++;
				return 1;
			}else{
				return 0;
			}
		}
	}

	return 0;
}

void rules_cleanup(int code, void *priv)
{
	p_parser foo,bar;

	print_raw("Removing parsers" CRLF);	
	if ( parsers == NULL ) return;

	CRITICAL_START(plock);
	for (foo=parsers; foo; )
	{
		bar=foo;
		foo=(p_parser)foo->l.next;
		globals.free(bar);
		bar=NULL;
	}
	CRITICAL_STOP(plock);
	
	return;
}

int rules_init(void)
{
	MUTEX_CREATE(plock, "Parsers");
	cleanup_add_handler(cleanup_lock, (void *)&plock);
	
	cleanup_add_handler(rules_cleanup, NULL);
	return 1;
}

int rules_add_parser(char *name, proc_parser_loadfile lf)
{
	p_parser p=NULL;

	if ( !name || !lf ) return 0;

	if ( !(p=globals.malloc(sizeof(parser_t))) ) return 0;

	p->name=name;
	p->loadfile=lf;
	
	CRITICAL_START(plock);
	parsers=list_append(p, parsers);
	CRITICAL_STOP(plock);
	
	return 1;
}
